#pragma once

DEFINE_GUID(CLSID_SimpleTriangleEffect, 0xef88e1e3, 0x3204, 0x4f56, 0x94, 0x98, 0x8a, 0xa6, 0xa4, 0x4b, 0xbb, 0x93);
DEFINE_GUID(GUID_SimpleTriangleEffectVertexShader, 0x7792717, 0x16c0, 0x452d, 0xba, 0x1, 0xa6, 0x1f, 0xd2, 0x9e, 0x96, 0x44);
DEFINE_GUID(GUID_SimpleTriangleEffectPixelShader, 0xdfe4e188, 0x523, 0x48d7, 0xb6, 0x2b, 0x83, 0xd2, 0xe, 0x49, 0x71, 0xb7);
DEFINE_GUID(GUID_SimpleTriangleEffectVertexBuffer, 0xb86b8305, 0x1b18, 0x4eb2, 0x8e, 0x1d, 0x22, 0xd1, 0x3, 0x85, 0x7a, 0xc4);

class SimpleTriangleEffect : public ID2D1EffectImpl, public ID2D1DrawTransform
{
private:
    LONG m_refCount;
    static std::vector<byte> m_vertexShaderData;
    static std::vector<byte> m_pixelShaderData;
    Microsoft::WRL::ComPtr<ID2D1VertexBuffer> m_vertexBuffer;
    D2D1_VERTEX_RANGE m_vertexRange;

    // Private constructor
    SimpleTriangleEffect();

public:
    static Concurrency::task<HRESULT> RegisterEffectAsync(_In_ ID2D1Factory1* pFactory);

private:
    static HRESULT __stdcall Create(_Outptr_ IUnknown** ppEffectImpl);

public:
    // IUnknown methods
    virtual ULONG STDMETHODCALLTYPE AddRef() override;
    virtual ULONG STDMETHODCALLTYPE Release() override;
    virtual HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, void **ppvObject) override;

    // ID2D1EffectImpl methods
    virtual HRESULT STDMETHODCALLTYPE Initialize(_In_ ID2D1EffectContext* pEffectContext, 
                                                 _In_ ID2D1TransformGraph* pTransformGraph) override;

    virtual HRESULT STDMETHODCALLTYPE PrepareForRender(D2D1_CHANGE_TYPE changeType) override;
    virtual HRESULT STDMETHODCALLTYPE SetGraph(_In_ ID2D1TransformGraph* pTransformGraph) override;

    // ID2D1TransformNode method
    virtual UINT32 STDMETHODCALLTYPE GetInputCount() const;

    // ID2D1Transform methods.
    virtual HRESULT STDMETHODCALLTYPE MapOutputRectToInputRects(
        _In_ const D2D1_RECT_L* pOutputRect,
        _Out_writes_(inputRectCount) D2D1_RECT_L* pInputRects,
        UINT32 inputRectCount
        ) const;

    virtual HRESULT STDMETHODCALLTYPE MapInputRectsToOutputRect(
        _In_reads_(inputRectCount) CONST D2D1_RECT_L *pInputRects,
        _In_reads_(inputRectCount) CONST D2D1_RECT_L *pInputOpaqueSubRects,
        UINT32 inputRectCount,
        _Out_ D2D1_RECT_L *pOutputRect,
        _Out_ D2D1_RECT_L *pOutputOpaqueSubRect
        ) override;

    virtual HRESULT STDMETHODCALLTYPE MapInvalidRect(
        UINT32 inputIndex,
        D2D1_RECT_L invalidInputRect,
        _Out_ D2D1_RECT_L *pInvalidOutputRect
        ) const;

    // ID2D1DrawTransform method
    virtual HRESULT STDMETHODCALLTYPE SetDrawInfo(_In_ ID2D1DrawInfo* pDrawInfo) override;
};