// Per-vertex data input to the vertex shader
struct VertexShaderInput
{
    float3 position : MESH_POSITION;
    float3 color : COLOR0;
};

// Per-vertex data output from the vertex shader
struct VertexShaderOutput
{
    float4 clipSpaceOutput : SV_POSITION;
    float4 sceneSpaceOutput : SCENE_POSITION;
    float3 color : COLOR0;
};

// Information provided for Direct2D vertex shaders
cbuffer ClipSpaceTransforms : register(b0)
{
    float2x1 sceneToOutputX;
    float2x1 sceneToOutputY;
}

// Called for each vertex.
VertexShaderOutput main(VertexShaderInput input)
{
    // Output structure
    VertexShaderOutput output;

    // Append a 'w' value of 1 to the 3D input position
    output.sceneSpaceOutput = float4(input.position.xyz, 1);
    
    // Standard calculations
    output.clipSpaceOutput.x = 
        output.sceneSpaceOutput.x * sceneToOutputX[0] +
        output.sceneSpaceOutput.w * sceneToOutputX[1];

    output.clipSpaceOutput.y = 
        output.sceneSpaceOutput.y * sceneToOutputY[0] +
        output.sceneSpaceOutput.w * sceneToOutputY[1];

    output.clipSpaceOutput.z = output.sceneSpaceOutput.z;
    output.clipSpaceOutput.w = output.sceneSpaceOutput.w;

    // Transfer the color
    output.color = input.color;

    return output;
}