﻿#include "pch.h"
#include "Common/DirectXHelper.h"
#include "ThreeTrianglesRenderer.h"

using namespace Concurrency;
using namespace D2D1;
using namespace Microsoft::WRL;

using namespace ThreeTriangles;

ThreeTrianglesRenderer::ThreeTrianglesRenderer(const std::shared_ptr<DX::DeviceResources>& deviceResources) :
    m_readyToRender(false),
    m_deviceResources(deviceResources)
{
    ID2D1Factory2* d2dFactory = m_deviceResources->GetD2DFactory();

    // Register the effect
    task<HRESULT> task =
        create_task(SimpleTriangleEffect::RegisterEffectAsync(d2dFactory));

    task.then([this](HRESULT hr)
    {
        DX::ThrowIfFailed(hr);
        CreateDeviceDependentResources();
    });
}

void ThreeTrianglesRenderer::CreateDeviceDependentResources()
{
    ID2D1DeviceContext1* d2dContext = m_deviceResources->GetD2DDeviceContext();

    // Create the effect
    DX::ThrowIfFailed(d2dContext->CreateEffect(
                    CLSID_SimpleTriangleEffect, &m_simpleTriangleEffect)
        );

    m_readyToRender = true;
}

void ThreeTrianglesRenderer::ReleaseDeviceDependentResources()
{
    m_simpleTriangleEffect.Reset();
}

void ThreeTrianglesRenderer::Update(DX::StepTimer const& timer)
{
}

// Renders a frame to the screen.
bool ThreeTrianglesRenderer::Render()
{
    if (!m_readyToRender)
        return false;

    ID2D1DeviceContext* d2dContext = m_deviceResources->GetD2DDeviceContext();
    Windows::Foundation::Size logicalSize = m_deviceResources->GetLogicalSize();

    d2dContext->BeginDraw();
    d2dContext->Clear(ColorF(ColorF::Gray));

    // Scale transform to scale 1000-pixel radius to screen size
    float dpiX, dpiY;
    d2dContext->GetDpi(&dpiX, &dpiY);
    float scale = (float) min(logicalSize.Width, 
                              logicalSize.Height) / 2 / 1000;
    scale *= dpiX / 96;
    Matrix3x2F scaleTransform = Matrix3x2F::Scale(scale, scale);

    // Use translate transform to shift to center
    Matrix3x2F centerTransform = 
        Matrix3x2F::Translation(logicalSize.Width / 2, logicalSize.Height / 2);

    d2dContext->SetTransform(scaleTransform * centerTransform *
                          m_deviceResources->GetOrientationTransform2D());

    d2dContext->DrawImage(m_simpleTriangleEffect.Get());

    // Ignore D2DERR_RECREATE_TARGET here. This error indicates that the device
    // is lost. It will be handled during the next call to Present.
    HRESULT hr = d2dContext->EndDraw();
    if (hr != D2DERR_RECREATE_TARGET)
    {
        DX::ThrowIfFailed(hr);
    }
    return true;
}
